@@ -7,7 +7,7 @@ GNU Classpath is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
- 
+
 GNU Classpath is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -64,7 +64,7 @@ import java.awt.Font;
  * package and then in the property editor search path.
  *
  * <p>Default property editors are provided for:</p>
- * 
+ *
  * <ol>
  * <li>boolean, byte, short, int, long, float, and double</li>
  * <li>java.lang.String</li>
@@ -83,7 +83,8 @@ import java.awt.Font;
 
 public class PropertyEditorManager
 {
-  static java.util.Hashtable editors = new java.util.Hashtable();
+  static java.util.Hashtable<Class<?>,Class<?>> editors =
+    new java.util.Hashtable<Class<?>,Class<?>>();
   static String[] editorSearchPath = { "gnu.java.beans.editors",
                                        "sun.beans.editors" };
 
@@ -118,7 +119,7 @@ public class PropertyEditorManager
    *        will edit.
    * @param editorClass the PropertyEditor class.
    */
-  public static void registerEditor(Class editedClass, Class editorClass)
+  public static void registerEditor(Class<?> editedClass, Class<?> editorClass)
   {
     editors.put(editedClass, editorClass);
   }
@@ -132,7 +133,7 @@ public class PropertyEditorManager
    * @return a PropertyEditor instance that can edit the
    *         specified class.
    */
-  public static PropertyEditor findEditor(Class editedClass)
+  public static PropertyEditor findEditor(Class<?> editedClass)
   {
     try
       {
@@ -142,13 +143,13 @@ public class PropertyEditorManager
             return (PropertyEditor)found.newInstance();
           }
 
-	ClassLoader contextClassLoader
-		= Thread.currentThread().getContextClassLoader();
+        ClassLoader contextClassLoader
+                = Thread.currentThread().getContextClassLoader();
 
         try
           {
             found = Class.forName(editedClass.getName()+"Editor", true,
-				  contextClassLoader);
+                                  contextClassLoader);
             registerEditor(editedClass,found);
             return (PropertyEditor)found.newInstance();
           }
@@ -157,9 +158,9 @@ public class PropertyEditorManager
           }
 
         String appendName
-		= "."
-		+ ClassHelper.getTruncatedClassName(editedClass)
-		+ "Editor";
+                = "."
+                + ClassHelper.getTruncatedClassName(editedClass)
+                + "Editor";
         synchronized(editorSearchPath)
           {
             for(int i=0;i<editorSearchPath.length;i++)
@@ -167,7 +168,7 @@ public class PropertyEditorManager
                 try
                   {
                     found = Class.forName(editorSearchPath[i] + appendName,
-					  true, contextClassLoader);
+                                          true, contextClassLoader);
                     registerEditor(editedClass,found);
                     return (PropertyEditor)found.newInstance();
                   }
@@ -183,7 +184,7 @@ public class PropertyEditorManager
     catch(IllegalAccessException E)
       {
       }
-    
+
     return null;
   }
 
