@@ -1,5 +1,5 @@
 /* StackTraceElement.java -- One function call or call stack element
-   Copyright (C) 2001, 2002, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2001, 2002, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -46,10 +46,10 @@ import java.io.Serializable;
  * the fully qualified class name, the method name and whether this method
  * is native, if this information is known.
  *
- * @author Mark Wielaard <mark@klomp.org>
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Mark Wielaard (mark@klomp.org)
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @since 1.4
- * @status updated to 1.4
+ * @status updated to 1.5
  */
 public final class StackTraceElement implements Serializable
 {
@@ -111,6 +111,26 @@ public final class StackTraceElement implements Serializable
     this.isNative = isNative;
   }
 
+  /**
+   * Create a new StackTraceElement representing a given source location.
+   *
+   * @param className the fully qualified name of the class
+   * @param methodName the name of the method
+   * @param fileName the name of the file, null if unknown
+   * @param lineNumber the line in the file, negative if unknown, or -2
+   * if this method is native
+   *
+   * @since 1.5
+   */
+  public StackTraceElement(String className, String methodName, String fileName,
+                           int lineNumber)
+  {
+    this(fileName, lineNumber, className, methodName, lineNumber == -2);
+    // The public constructor doesn't allow certain values to be null.
+    if (className == null || methodName == null)
+      throw new NullPointerException("invalid argument to constructor");
+  }
+
   /**
    * Returns the name of the file, or null if unknown. This is usually
    * obtained from the <code>SourceFile</code> attribute of the class file
@@ -182,7 +202,7 @@ public final class StackTraceElement implements Serializable
    */
   public String toString()
   {
-    StringBuffer sb = new StringBuffer();
+    StringBuilder sb = new StringBuilder();
     if (declaringClass != null)
       {
         sb.append(declaringClass);
@@ -191,7 +211,7 @@ public final class StackTraceElement implements Serializable
       }
     if (methodName != null)
       sb.append(methodName);
-    sb.append(" (");
+    sb.append("(");
     if (fileName != null)
       sb.append(fileName);
     else
