@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package javax.swing;
 
+import gnu.java.lang.CPStringBuilder;
+
 import java.awt.Component;
 import java.awt.Graphics;
 import java.awt.Insets;
@@ -59,14 +61,14 @@ import javax.swing.border.Border;
  * components to container, one can use add() to add menu's to the menu bar.
  * Menu's will be displayed in the menu  bar in the order they were added.
  * The JMenuBar uses selectionModel to keep track of selected menu index.
- * JMenuBar's selectionModel will fire ChangeEvents to its registered 
+ * JMenuBar's selectionModel will fire ChangeEvents to its registered
  * listeners when the selected index changes.
  */
 public class JMenuBar extends JComponent implements Accessible, MenuElement
 {
   /**
    * Provides accessibility support for <code>JMenuBar</code>.
-   * 
+   *
    * @author Roman Kennke (kennke@aicas.com)
    */
   protected class AccessibleJMenuBar extends AccessibleJComponent
@@ -287,30 +289,36 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
 
     for (int i = 0; i < comps.length; i++)
       {
-	if (comps[i].equals(c))
-	  {
-	    index = i;
-	    break;
-	  }
+        if (comps[i].equals(c))
+          {
+            index = i;
+            break;
+          }
       }
 
     return index;
   }
 
   /**
-   * DOCUMENT ME!
+   * This method is not implemented and will throw an {@link Error} if called.
    *
-   * @return DOCUMENT ME!
+   * @return This method never returns anything, it throws an exception.
    */
   public JMenu getHelpMenu()
   {
-    return null;
+    // the following error matches the behaviour of the reference
+    // implementation...
+    throw new Error("getHelpMenu() is not implemented");
   }
 
   /**
-   * Returns margin betweeen menu bar's border and its menues
+   * Returns the margin between the menu bar's border and its menus.  If the
+   * margin is <code>null</code>, this method returns
+   * <code>new Insets(0, 0, 0, 0)</code>.
    *
-   * @return margin between menu bar's border and its menues
+   * @return The margin (never <code>null</code>).
+   *
+   * @see #setMargin(Insets)
    */
   public Insets getMargin()
   {
@@ -370,10 +378,30 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
   {
     MenuElement[] subElements = new MenuElement[getComponentCount()];
 
+    int j = 0;
+    boolean doResize = false;
+    MenuElement menu;
     for (int i = 0; i < getComponentCount(); i++)
-      subElements[i] = (MenuElement) getMenu(i);
+      {
+        menu = getMenu(i);
+        if (menu != null)
+          {
+            subElements[j++] = (MenuElement) menu;
+          }
+        else
+          doResize = true;
+      }
 
-    return subElements;
+    if (! doResize)
+      return subElements;
+    else
+      {
+        MenuElement[] subElements2 = new MenuElement[j];
+        for (int i = 0; i < j; i++)
+          subElements2[i] = subElements[i];
+
+        return subElements2;
+      }
   }
 
   /**
@@ -455,7 +483,7 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
    */
   protected String paramString()
   {
-    StringBuffer sb = new StringBuffer();
+    CPStringBuilder sb = new CPStringBuilder();
     sb.append(super.paramString());
     sb.append(",margin=");
     if (getMargin() != null)
@@ -480,14 +508,14 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
   }
 
   /**
-   * This method overrides JComponent.processKeyBinding to allow the 
-   * JMenuBar to check all the child components (recursiveley) to see 
+   * This method overrides JComponent.processKeyBinding to allow the
+   * JMenuBar to check all the child components (recursiveley) to see
    * if they'll consume the event.
-   * 
+   *
    * @param ks the KeyStroke for the event
    * @param e the KeyEvent for the event
    * @param condition the focus condition for the binding
-   * @param pressed true if the key is pressed 
+   * @param pressed true if the key is pressed
    */
   protected boolean processKeyBinding(KeyStroke ks, KeyEvent e, int condition,
                                       boolean pressed)
@@ -495,19 +523,19 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
     // See if the regular JComponent behavior consumes the event
     if (super.processKeyBinding(ks, e, condition, pressed))
       return true;
-    
-    // If not, have to recursively check all the child menu elements to see 
-    // if they want it    
+
+    // If not, have to recursively check all the child menu elements to see
+    // if they want it
     MenuElement[] children = getSubElements();
     for (int i = 0; i < children.length; i++)
       if (processKeyBindingHelper(children[i], ks, e, condition, pressed))
         return true;
     return false;
   }
-  
+
   /**
    * This is a helper method to recursively check the children of this
-   * JMenuBar to see if they will consume a key event via key bindings.  
+   * JMenuBar to see if they will consume a key event via key bindings.
    * This is used for menu accelerators.
    * @param menuElement the menuElement to check (and check all its children)
    * @param ks the KeyStroke for the event
@@ -522,12 +550,15 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
                                          KeyEvent e, int condition,
                                          boolean pressed)
   {
+    if (menuElement == null)
+      return false;
+
     // First check the menuElement itself, if it's a JComponent
     if (menuElement instanceof JComponent
         && ((JComponent) menuElement).processKeyBinding(ks, e, condition,
                                                         pressed))
       return true;
-    
+
     // If that didn't consume it, check all the children recursively
     MenuElement[] children = menuElement.getSubElements();
     for (int i = 0; i < children.length; i++)
@@ -535,7 +566,7 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
         return true;
     return false;
   }
-  
+
   /**
    * Process mouse events forwarded from MenuSelectionManager. This method
    * doesn't do anything. It is here to conform to the MenuElement interface.
@@ -571,11 +602,11 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
   {
     if (b != borderPainted)
       {
-	boolean old = borderPainted;
-	borderPainted = b;
-	firePropertyChange("borderPainted", old, b);
-	revalidate();
-	repaint();
+        boolean old = borderPainted;
+        borderPainted = b;
+        firePropertyChange("borderPainted", old, b);
+        revalidate();
+        repaint();
       }
   }
 
@@ -594,21 +625,20 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
   }
 
   /**
-   * Sets the menu bar's "margin" bound property,  which represents
-   * distance between the menubar's border and its menus.
-   * icon. When marging property is modified, PropertyChangeEvent will
-   * be fired to menuBar's PropertyChangeListener's.
+   * Sets the margin between the menu bar's border and its menus (this is a
+   * bound property with the name 'margin').
    *
-   * @param m distance between the menubar's border and its menus.
+   * @param m  the margin (<code>null</code> permitted).
    *
+   * @see #getMargin()
    */
   public void setMargin(Insets m)
   {
     if (m != margin)
       {
-	Insets oldMargin = margin;
-	margin = m;
-	firePropertyChange("margin", oldMargin, margin);
+        Insets oldMargin = margin;
+        margin = m;
+        firePropertyChange("margin", oldMargin, margin);
       }
   }
 
@@ -634,9 +664,9 @@ public class JMenuBar extends JComponent implements Accessible, MenuElement
   {
     if (selectionModel != model)
       {
-	SingleSelectionModel oldModel = selectionModel;
-	selectionModel = model;
-	firePropertyChange("model", oldModel, selectionModel);
+        SingleSelectionModel oldModel = selectionModel;
+        selectionModel = model;
+        firePropertyChange("model", oldModel, selectionModel);
       }
   }
 
