@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -39,17 +39,33 @@ exception statement from your version. */
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkScrollPanePeer.h"
 
+#define AWT_SCROLLPANE_SCROLLBARS_AS_NEEDED 0
+#define AWT_SCROLLPANE_SCROLLBARS_ALWAYS 1
+#define AWT_SCROLLPANE_SCROLLBARS_NEVER 2
+
+static GtkWidget *scrollpane_get_widget (GtkWidget *widget);
+
 JNIEXPORT void JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_create 
-  (JNIEnv *env, jobject obj)
+  (JNIEnv *env, jobject obj, int width, int height)
 {
-  gpointer window;
-
+  GtkWidget *sw;
+  GtkWidget *eventbox;
+  
   gdk_threads_enter ();
-  window = gtk_scrolled_window_new (NULL, NULL);
-  gdk_threads_leave ();
+  
+  /* Create global reference and save it for future use */
+  gtkpeer_set_global_ref (env, obj);
 
-  NSA_SET_PTR (env, obj, window);
+  sw = gtk_scrolled_window_new (NULL, NULL);
+  gtk_widget_set_size_request (sw, width, height);
+  eventbox = gtk_event_box_new ();
+  gtk_container_add (GTK_CONTAINER (eventbox), sw);
+  gtk_widget_show (sw);
+
+  gtkpeer_set_widget (env, obj, eventbox);
+
+  gdk_threads_leave ();
 }
 
 JNIEXPORT void JNICALL 
@@ -60,16 +76,18 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_setScrollPosition
   GtkScrolledWindow *sw;
   void *ptr;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
-  sw = GTK_SCROLLED_WINDOW (ptr);
+
+  ptr = gtkpeer_get_widget (env, obj);
+
+  sw = GTK_SCROLLED_WINDOW (scrollpane_get_widget (GTK_WIDGET (ptr)));
 
   hadj = gtk_scrolled_window_get_hadjustment (sw);
   vadj = gtk_scrolled_window_get_vadjustment (sw);
 
   gtk_adjustment_set_value (hadj, x);
   gtk_adjustment_set_value (vadj, y);
+
   gdk_threads_leave ();
 }
 
@@ -81,10 +99,11 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_gtkScrolledWindowSetHScrollIncremen
   GtkScrolledWindow *sw;
   void *ptr;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
-  sw = GTK_SCROLLED_WINDOW(ptr);
+
+  ptr = gtkpeer_get_widget (env, obj);
+
+  sw = GTK_SCROLLED_WINDOW (scrollpane_get_widget (GTK_WIDGET (ptr)));
 
   hadj = gtk_scrolled_window_get_hadjustment (sw);
   hadj->step_increment = u;
@@ -100,10 +119,11 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_gtkScrolledWindowSetVScrollIncremen
   GtkScrolledWindow *sw;
   void *ptr;
 
-  ptr = NSA_GET_PTR (env, obj);
-
   gdk_threads_enter ();
-  sw = GTK_SCROLLED_WINDOW(ptr);
+
+  ptr = gtkpeer_get_widget (env, obj);
+
+  sw = GTK_SCROLLED_WINDOW (scrollpane_get_widget (GTK_WIDGET (ptr)));
 
   vadj = gtk_scrolled_window_get_hadjustment (sw);
   vadj->step_increment = u;
@@ -111,34 +131,26 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_gtkScrolledWindowSetVScrollIncremen
   gdk_threads_leave ();
 }
 
-JNIEXPORT void JNICALL
-Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_childResized
-  (JNIEnv *env, jobject obj, jint width, jint height)
-{
-  void *ptr;
-
-  ptr = NSA_GET_PTR (env, obj);
-
-  return;
-
-  gdk_threads_enter ();
-  gtk_widget_set_usize (GTK_BIN (ptr)->child, width, height);
-  gdk_threads_leave ();
-}
-
 JNIEXPORT jint JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_getHScrollbarHeight
   (JNIEnv *env, jobject obj)
 {
   void *ptr;
   GtkScrolledWindow *sw;
-  jint height;
-
-  ptr = NSA_GET_PTR (env, obj);
+  GtkRequisition requisition;
+  jint height = 0;
+  jint spacing = 0;
 
   gdk_threads_enter ();
-  sw = GTK_SCROLLED_WINDOW (ptr);
-  height = (sw->hscrollbar_visible) ? sw->hscrollbar->allocation.height : 0;
+
+  ptr = gtkpeer_get_widget (env, obj);
+
+  sw = GTK_SCROLLED_WINDOW (scrollpane_get_widget (GTK_WIDGET (ptr)));
+
+  gtk_widget_size_request (sw->hscrollbar, &requisition);
+  gtk_widget_style_get (GTK_WIDGET (sw), "scrollbar_spacing", &spacing, NULL);
+  height = requisition.height + spacing;
+
   gdk_threads_leave ();
 
   return height;
@@ -150,13 +162,20 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_getVScrollbarWidth
 {
   void *ptr;
   GtkScrolledWindow *sw;
-  jint width;
-
-  ptr = NSA_GET_PTR (env, obj);
+  GtkRequisition requisition;
+  jint width = 0;
+  jint spacing = 0;
 
   gdk_threads_enter ();
-  sw = GTK_SCROLLED_WINDOW (ptr);
-  width = (sw->vscrollbar_visible) ? sw->vscrollbar->allocation.width : 0;
+
+  ptr = gtkpeer_get_widget (env, obj);
+
+  sw = GTK_SCROLLED_WINDOW (scrollpane_get_widget (GTK_WIDGET (ptr)));
+
+  gtk_widget_size_request (sw->vscrollbar, &requisition);
+  gtk_widget_style_get (GTK_WIDGET (sw), "scrollbar_spacing", &spacing, NULL);
+  width = requisition.width + spacing;
+
   gdk_threads_leave ();
 
   return width;
@@ -167,8 +186,11 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_setPolicy
   (JNIEnv *env, jobject obj, jint policy)
 {
   void *ptr;
+  GtkWidget *sw;
+  
+  gdk_threads_enter ();
 
-  ptr = NSA_GET_PTR (env, obj);
+  ptr = gtkpeer_get_widget (env, obj);
 
   switch (policy)
     {
@@ -183,7 +205,18 @@ Java_gnu_java_awt_peer_gtk_GtkScrollPanePeer_setPolicy
       break;
     }
 
-  gdk_threads_enter ();
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ptr), policy, policy);
+  sw = scrollpane_get_widget (GTK_WIDGET (ptr));
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), policy, policy);
+
   gdk_threads_leave ();
 }
+
+static GtkWidget *
+scrollpane_get_widget (GtkWidget *widget)
+{
+  GtkWidget *wid;
+  g_assert (GTK_IS_EVENT_BOX (widget));
+  wid = gtk_bin_get_child (GTK_BIN(widget));
+
+  return wid;
+}
