@@ -38,7 +38,8 @@ exception statement from your version. */
 
 package org.omg.CORBA;
 
-import gnu.CORBA.Restricted_ORB;
+import gnu.CORBA.Minor;
+import gnu.CORBA.OrbRestricted;
 import gnu.CORBA.TypeCodeHelper;
 
 import org.omg.CORBA.portable.InputStream;
@@ -91,7 +92,7 @@ public abstract class ValueMemberHelper
           {
             if (typeCode == null)
               {
-                ORB orb = Restricted_ORB.Singleton;
+                ORB orb = OrbRestricted.Singleton;
 
                 if (active)
                   {
@@ -177,6 +178,7 @@ public abstract class ValueMemberHelper
     catch (UserException ex)
       {
         MARSHAL m = new MARSHAL();
+        m.minor = Minor.UserException;
         m.initCause(ex);
         throw m;
       }
@@ -200,8 +202,9 @@ public abstract class ValueMemberHelper
     catch (UserException ex)
       {
         MARSHAL m = new MARSHAL();
+        m.minor = Minor.UserException;
         m.initCause(ex);
         throw m;
       }
   }
-}
\ No newline at end of file
+}
