@@ -1,6 +1,7 @@
 /* VMClassLoader.java -- Reference implementation of native interface
    required by ClassLoader
-   Copyright (C) 1998, 2001, 2002, 2004, 2005, 2006 Free Software Foundation
+   Copyright (C) 1998, 2001, 2002, 2004, 2005, 2006, 2010
+   Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -71,10 +72,12 @@ import java.util.zip.ZipFile;
 final class VMClassLoader
 {
 
-
   /** packages loaded by the bootstrap class loader */
   static final HashMap definedPackages = new HashMap();
 
+  /** jars from property java.boot.class.path */
+  static final HashMap bootjars = new HashMap();
+
   /**
    * Converts the array string of native package names to
    * Packages. The packages are then put into the
@@ -83,18 +86,18 @@ final class VMClassLoader
   static
   {
     String[] packages = getBootPackages();
-    
+
     if( packages != null)
       {
-        String specName = 
+        String specName =
               SystemProperties.getProperty("java.specification.name");
         String vendor =
               SystemProperties.getProperty("java.specification.vendor");
         String version =
               SystemProperties.getProperty("java.specification.version");
-        
+
         Package p;
-              
+
         for(int i = 0; i < packages.length; i++)
           {
             p = new Package(packages[i],
@@ -112,7 +115,8 @@ final class VMClassLoader
       }
   }
 
-  
+  private VMClassLoader() {} // Prohibits instantiation.
+
   /**
    * Helper to define a class using a string of bytes. This assumes that
    * the security checks have already been performed, if necessary.
@@ -169,9 +173,6 @@ final class VMClassLoader
     return null;
   }
 
-  /** jars from property java.boot.class.path */
-  static final HashMap bootjars = new HashMap();
-  
   /**
    * Helper to get a list of resources from the bootstrap class loader.
    *
@@ -186,23 +187,23 @@ final class VMClassLoader
     Vector v = new Vector();
     while (st.hasMoreTokens())
       {
-	File file = new File(st.nextToken());
-	if (file.isDirectory())
-	  {
-	    try
-	      {
+        File file = new File(st.nextToken());
+        if (file.isDirectory())
+          {
+            try
+              {
                 File f = new File(file, name);
                 if (!f.exists()) continue;
                 v.add(new URL("file://" + f.getAbsolutePath()));
-	      }
-	    catch (MalformedURLException e)
-	      {
-		throw new Error(e);
-	      }
-	  }
-	else if (file.isFile())
-	  {
-	    ZipFile zip;
+              }
+            catch (MalformedURLException e)
+              {
+                throw new Error(e);
+              }
+          }
+        else if (file.isFile())
+          {
+            ZipFile zip;
             synchronized(bootjars)
               {
                 zip = (ZipFile) bootjars.get(file.getName());
@@ -210,36 +211,35 @@ final class VMClassLoader
             if(zip == null)
               {
                 try
-	          {
+                  {
                     zip = new ZipFile(file);
                     synchronized(bootjars)
                       {
                         bootjars.put(file.getName(), zip);
                       }
-	          }
-	        catch (IOException e)
-	          {
-		    continue;
-	          }
+                  }
+                catch (IOException e)
+                  {
+                    continue;
+                  }
               }
-	    String zname = name.startsWith("/") ? name.substring(1) : name;
-	    if (zip.getEntry(zname) == null)
-	      continue;
-	    try
-	      {
-		v.add(new URL("jar:file://"
-		  + file.getAbsolutePath() + "!/" + zname));
-	      }
-	    catch (MalformedURLException e)
-	      {
-		throw new Error(e);
-	      }
-	  }
+            String zname = name.startsWith("/") ? name.substring(1) : name;
+            if (zip.getEntry(zname) == null)
+              continue;
+            try
+              {
+                v.add(new URL("jar:file://"
+                  + file.getAbsolutePath() + "!/" + zname));
+              }
+            catch (MalformedURLException e)
+              {
+                throw new Error(e);
+              }
+          }
       }
     return v.elements();
   }
 
-
   /**
    * Returns a String[] of native package names. The default
    * implementation tries to load a list of package from
@@ -284,7 +284,6 @@ final class VMClassLoader
       return new String[0];
   }
 
-
   /**
    * Helper to get a package from the bootstrap class loader.
    *
@@ -297,9 +296,9 @@ final class VMClassLoader
   }
 
 
-  
+
   /**
-   * Helper to get all packages from the bootstrap class loader.  
+   * Helper to get all packages from the bootstrap class loader.
    *
    * @return all named packages, if any exist
    */
@@ -410,15 +409,16 @@ final class VMClassLoader
   static final Class defineClassWithTransformers(ClassLoader loader,
       String name, byte[] data, int offset, int len, ProtectionDomain pd)
   {
-    
+
     if (instrumenter != null)
       {
         byte[] modifiedData = new byte[len];
         System.arraycopy(data, offset, modifiedData, 0, len);
+        String jvmName = name.replace('.', '/');
         modifiedData =
-          ((InstrumentationImpl)instrumenter).callTransformers(loader, name,
+          ((InstrumentationImpl)instrumenter).callTransformers(loader, jvmName,
             null, pd, modifiedData);
-        
+
         return defineClass(loader, name, modifiedData, 0, modifiedData.length,
             pd);
       }
